@@ -2,11 +2,13 @@
 # MIT License (see LICENSE or https://opensource.org/licenses/MIT)
 
 import base64
+import copy
 import logging
 import sys
 import typing
 
 from spnego._context import (
+    IOV,
     ContextProxy,
     ContextReq,
     GSSMech,
@@ -17,11 +19,19 @@ from spnego._context import (
     WrapResult,
     wrap_system_error,
 )
+from spnego._credential import (
+    Credential,
+    CredentialCache,
+    KerberosCCache,
+    KerberosKeytab,
+    Password,
+    unify_credentials,
+)
 from spnego._text import to_bytes, to_text
 from spnego.channel_bindings import GssChannelBindings
 from spnego.exceptions import GSSError as NativeError
-from spnego.exceptions import NegotiateOptions, SpnegoError
-from spnego.iov import BufferType, IOVBuffer
+from spnego.exceptions import InvalidCredentialError, NegotiateOptions, SpnegoError
+from spnego.iov import BufferType, IOVBuffer, IOVResBuffer
 
 log = logging.getLogger(__name__)
 
@@ -42,7 +52,9 @@ except ImportError as e:
 HAS_IOV = True
 GSSAPI_IOV_IMP_ERR = None
 try:
-    from gssapi.raw import IOV, IOVBufferType, unwrap_iov, wrap_iov
+    from gssapi.raw import IOV as GSSIOV
+    from gssapi.raw import IOVBuffer as GSSIOVBuffer
+    from gssapi.raw import IOVBufferType, unwrap_iov, wrap_iov
 except ImportError as err:
     GSSAPI_IOV_IMP_ERR = sys.exc_info()
     HAS_IOV = False
@@ -51,14 +63,14 @@ except ImportError as err:
 _GSS_C_INQ_SSPI_SESSION_KEY = "1.2.840.113554.1.2.2.5.5"
 
 # https://github.com/simo5/gss-ntlmssp/blob/bfc7232dbb2259072a976fc9cdb6ae4bfd323304/src/gssapi_ntlmssp.h#L68
-_GSS_NTLMSSP_RESET_CRYPTO_OID = '1.3.6.1.4.1.7165.655.1.3'
+_GSS_NTLMSSP_RESET_CRYPTO_OID = "1.3.6.1.4.1.7165.655.1.3"
 
 # https://github.com/krb5/krb5/blob/master/src/lib/gssapi/spnego/spnego_mech.c#L483
-_GSS_SPNEGO_REQUIRE_MIC_OID_STRING = '1.3.6.1.4.1.7165.655.1.2'
+_GSS_SPNEGO_REQUIRE_MIC_OID_STRING = "1.3.6.1.4.1.7165.655.1.2"
 
 
 def _available_protocols(options: typing.Optional[NegotiateOptions] = None) -> typing.List[str]:
-    """ Return a list of protocols that GSSAPIProxy can offer. """
+    """Return a list of protocols that GSSAPIProxy can offer."""
     if not options:
         options = NegotiateOptions(0)
 
@@ -66,24 +78,24 @@ def _available_protocols(options: typing.Optional[NegotiateOptions] = None) -> t
     if HAS_GSSAPI:
         # We can't offer Kerberos if the caller requires WinRM wrapping and IOV isn't available.
         if not (options & NegotiateOptions.wrapping_winrm and not HAS_IOV):
-            protocols = ['kerberos']
+            protocols = ["kerberos"]
 
         # We can only offer NTLM if the mech is installed and can retrieve the functionality the caller desires.
         if _gss_ntlmssp_available(session_key=bool(options & NegotiateOptions.session_key)):
-            protocols.append('ntlm')
+            protocols.append("ntlm")
 
         # We can only offer Negotiate if we can offer both Kerberos and NTLM.
         if len(protocols) == 2:
-            protocols.append('negotiate')
+            protocols.append("negotiate")
 
     return protocols
 
 
-def _create_iov_result(iov: "IOV") -> typing.Tuple[IOVBuffer, ...]:
-    """ Converts GSSAPI IOV buffer to generic IOVBuffer result. """
+def _create_iov_result(iov: "GSSIOV") -> typing.Tuple[IOVResBuffer, ...]:
+    """Converts GSSAPI IOV buffer to generic IOVBuffer result."""
     buffers = []
     for i in iov:
-        buffer_entry = IOVBuffer(type=BufferType(i.type), data=i.value)
+        buffer_entry = IOVResBuffer(type=BufferType(i.type), data=i.value)
         buffers.append(buffer_entry)
 
     return tuple(buffers)
@@ -92,15 +104,14 @@ def _create_iov_result(iov: "IOV") -> typing.Tuple[IOVBuffer, ...]:
 def _get_gssapi_credential(
     mech: "gssapi.OID",
     usage: str,
-    username: typing.Optional[str] = None,
-    password: typing.Optional[str] = None,
+    credentials: typing.List[Credential],
     context_req: typing.Optional[ContextReq] = None,
 ) -> typing.Optional["gssapi.creds.Credentials"]:
-    """Gets a set of credential(s).
+    """Gets the GSSAPI credential.
 
-    Will get a set of GSSAPI credential(s) for the mech specified. If the username and password is specified then a new
-    set of credentials are explicitly required for the mech specified. Otherwise the credentials are retrieved by the
-    cache as defined by the mech.
+    Will get a GSSAPI credential for the mech specified. If the username and password is specified then a new
+    set of credentials are explicitly required for the mech specified. Otherwise the credentials are retrieved based on
+    the credential type specified.
 
     The behaviour of this function is highly dependent on the GSSAPI implementation installed as well as what NTLM mech
     is available. Here are some of the known behaviours of each mech.
@@ -121,8 +132,7 @@ def _get_gssapi_credential(
     Args:
         mech: The mech OID to get the credentials for.
         usage: Either `initiate` for a client context or `accept` for a server context.
-        username: The username to get the credentials for, if omitted then the default user is gotten from the cache.
-        password: The password for the user, if omitted then the cached credentials is retrieved.
+        credentials: List of credentials to retreive from.
         context_req: Context requirement flags that can control how the credential is retrieved.
 
     Returns:
@@ -131,39 +141,86 @@ def _get_gssapi_credential(
     .. _gss-ntlmssp:
         https://github.com/gssapi/gss-ntlmssp
     """
-    principal = None
-    if username:
-        name_type = getattr(gssapi.NameType, 'user' if usage == 'initiate' else 'hostbased_service')
-        principal = gssapi.Name(base=username, name_type=name_type)
+    supported_cred_types: typing.List[typing.Type[Credential]] = [Password, CredentialCache]
+    if mech.dotted_form == GSSMech.kerberos:
+        supported_cred_types.insert(0, KerberosCCache)
+        supported_cred_types.insert(1, KerberosKeytab)
 
-    if principal and password:
-        if usage == "initiate" and mech == gssapi.OID.from_int_seq(GSSMech.kerberos.value):
-            # GSSAPI offers no way to specify custom flags like forwardable when getting a Kerberos credential. This
-            # calls the Kerberos API to get the ticket and convert it to a GSSAPI credential.
-            forwardable = bool(
-                context_req and (context_req & ContextReq.delegate or context_req & ContextReq.delegate_policy)
+    kerberos_mech = gssapi.OID.from_int_seq(GSSMech.kerberos.value)
+    name_type = getattr(gssapi.NameType, "user" if usage == "initiate" else "hostbased_service")
+    forwardable = bool(context_req and (context_req & ContextReq.delegate or context_req & ContextReq.delegate_policy))
+
+    for cred in credentials:
+        if not isinstance(cred, tuple(supported_cred_types)):
+            log.debug("Skipping credential %s as it does not support required mech type", cred)
+            continue
+
+        if isinstance(cred, Password):
+            principal = gssapi.Name(base=cred.username, name_type=name_type)
+
+            if usage == "initiate" and mech == kerberos_mech:
+                # GSSAPI offers no way to specify custom flags like forwardable when getting a Kerberos credential.
+                # This calls the Kerberos API to get the ticket and convert it to a GSSAPI credential.
+                raw_cred = _kinit(to_bytes(cred.username), to_bytes(cred.password), forwardable=forwardable)
+
+            else:
+                # MIT krb5 < 1.14 would store the kerb cred in the global cache but later versions used a private cache
+                # in memory. There's not much we can do about this but document this behaviour and hope people upgrade
+                # to a newer version.
+                raw_cred = acquire_cred_with_password(
+                    principal, to_bytes(cred.password), usage=usage, mechs=[mech]
+                ).creds
+
+            return gssapi.Credentials(base=raw_cred, usage=usage)
+
+        elif isinstance(cred, CredentialCache):
+            principal = None
+            if cred.username:
+                principal = gssapi.Name(base=cred.username, name_type=name_type)
+
+            elif usage == "initiate":
+                # https://github.com/jborean93/pyspnego/issues/15
+                # Using None as a credential when creating the sec context is better than getting the default
+                # credential as the former takes into account the target SPN when selecting the principal to use.
+                return None
+
+            gss_cred = gssapi.Credentials(name=principal, usage=usage, mechs=[mech])
+
+            # We don't need to check the actual lifetime, just trying to get the valid will have gssapi check the
+            # lifetime and raise an ExpiredCredentialsError if it is expired.
+            _ = gss_cred.lifetime
+
+            return gss_cred
+
+        elif isinstance(cred, KerberosCCache):
+            if mech != kerberos_mech or usage != "initiate":
+                log.debug("Skipping %s as it can only be used for an initiate Kerberos context", cred)
+                continue
+
+            ctx = krb5.init_context()
+            ccache = krb5.cc_resolve(ctx, to_bytes(cred.ccache))
+            krb5_principal: typing.Optional[krb5.Principal] = None
+            if cred.principal:
+                krb5_principal = krb5.parse_name_flags(ctx, to_bytes(cred.principal))
+
+            return gssapi.Credentials(base=_gss_acquire_cred_from_ccache(ccache, krb5_principal), usage=usage)
+
+        elif isinstance(cred, KerberosKeytab):
+            if mech != kerberos_mech or usage != "initiate":
+                log.debug("Skipping %s as it can only be used for an initiate Kerberos context", cred)
+                continue
+
+            return gssapi.Credentials(
+                base=_kinit(
+                    to_bytes(cred.principal or b""),
+                    to_bytes(cred.keytab),
+                    forwardable=forwardable,
+                    is_keytab=True,
+                ),
+                usage=usage,
             )
-            cred = _kinit(to_bytes(username), to_bytes(password), forwardable=forwardable)
-        else:
-            # MIT krb5 < 1.14 would store the kerb cred in the global cache but later versions used a private cache in
-            # memory. There's not much we can do about this but document this behaviour and hope people upgrade to a
-            # newer version.
-            cred = acquire_cred_with_password(principal, to_bytes(password), usage=usage, mechs=[mech]).creds
 
-    elif principal or usage == 'accept':
-        cred = gssapi.Credentials(name=principal, usage=usage, mechs=[mech])
-
-        # We don't need to check the actual lifetime, just trying to get the valid will have gssapi check the lifetime
-        # and raise an ExpiredCredentialsError if it is expired.
-        _ = cred.lifetime
-
-    else:
-        # https://github.com/jborean93/pyspnego/issues/15
-        # Using None as a credential when creating the sec context is better than getting the default credential as the
-        # former takes into account the target SPN when selecting the principal to use.
-        cred = None
-
-    return cred
+    raise InvalidCredentialError(context_msg="No applicable credentials available")
 
 
 def _gss_ntlmssp_available(session_key: bool = False) -> bool:
@@ -204,13 +261,13 @@ def _gss_ntlmssp_available(session_key: bool = False) -> bool:
     # Cache the result so we don't run this check multiple times.
     try:
         res = _gss_ntlmssp_available.result  # type: ignore
-        return res['session_key'] if session_key else res['available']
+        return res["session_key"] if session_key else res["available"]
     except AttributeError:
         pass
 
     ntlm_features = {
-        'available': False,
-        'session_key': False,
+        "available": False,
+        "session_key": False,
     }
 
     # If any of these calls results in a GSSError we treat that as NTLM being unusable because these are standard
@@ -218,9 +275,9 @@ def _gss_ntlmssp_available(session_key: bool = False) -> bool:
     ntlm = gssapi.OID.from_int_seq(GSSMech.ntlm.value)
     try:
         # This can be anything, the first NTLM message doesn't need a valid target name or credential.
-        spn = gssapi.Name('http@test', name_type=gssapi.NameType.hostbased_service)
-        cred = _get_gssapi_credential(ntlm, 'initiate', username='user', password='pass')
-        context = gssapi.SecurityContext(creds=cred, usage='initiate', name=spn, mech=ntlm)
+        spn = gssapi.Name("http@test", name_type=gssapi.NameType.hostbased_service)
+        cred = _get_gssapi_credential(ntlm, "initiate", [Password(username="user", password="pass")])
+        context = gssapi.SecurityContext(creds=cred, usage="initiate", name=spn, mech=ntlm)
 
         context.step()  # Need to at least have a context set up before we can call gss_set_sec_context_option.
 
@@ -228,7 +285,7 @@ def _gss_ntlmssp_available(session_key: bool = False) -> bool:
         # it does not implement 'GSS_NTLMSSP_RESET_CRYPTO_OID' so by running this we can weed out that broken impl.
         _gss_ntlmssp_reset_crypto(context)
 
-        ntlm_features['available'] = True
+        ntlm_features["available"] = True
     except GSSError as gss_err:
         log.debug("GSSAPI does not support required the NTLM interfaces: %s" % str(gss_err))
     else:
@@ -242,8 +299,8 @@ def _gss_ntlmssp_available(session_key: bool = False) -> bool:
             # the context is not yet established. Any other errors would mean this isn't supported and we can't use
             # the current version installed if we need session_key interrogation.
             # https://github.com/gssapi/gss-ntlmssp/blob/9d7a275a4d6494606fb54713876e4f5cbf4d1362/src/gss_sec_ctx.c#L1277
-            if getattr(o_err, 'min_code', 0) == 1314127894:  # ERR_NOTAVAIL
-                ntlm_features['session_key'] = True
+            if getattr(o_err, "min_code", 0) == 1314127894:  # ERR_NOTAVAIL
+                ntlm_features["session_key"] = True
 
             else:
                 log.debug("GSSAPI ntlmssp does not support session key interrogation: %s" % str(o_err))
@@ -253,20 +310,20 @@ def _gss_ntlmssp_available(session_key: bool = False) -> bool:
 
 
 def _gss_ntlmssp_reset_crypto(context: "gssapi.SecurityContext", outgoing: bool = True) -> None:
-    """ Resets the NTLM RC4 ciphers when being used with SPNEGO. """
+    """Resets the NTLM RC4 ciphers when being used with SPNEGO."""
     reset_crypto = gssapi.OID.from_int_seq(_GSS_NTLMSSP_RESET_CRYPTO_OID)
     value = b"\x00\x00\x00\x00" if outgoing else b"\x01\x00\x00\x00"
     set_sec_context_option(reset_crypto, context=context, value=value)
 
 
 def _gss_sasl_description(mech: "gssapi.OID") -> typing.Optional[bytes]:
-    """ Attempts to get the SASL description of the mech specified. """
+    """Attempts to get the SASL description of the mech specified."""
     try:
         res = _gss_sasl_description.result  # type: ignore
         return res[mech.dotted_form]
 
     except (AttributeError, KeyError):
-        res = getattr(_gss_sasl_description, 'result', {})
+        res = getattr(_gss_sasl_description, "result", {})
 
     try:
         sasl_desc = gssapi.raw.inquire_saslname_for_mech(mech).mech_description
@@ -283,24 +340,42 @@ def _kinit(
     username: bytes,
     password: bytes,
     forwardable: typing.Optional[bool] = None,
+    is_keytab: bool = False,
 ) -> "gssapi.raw.Creds":
     """Gets a Kerberos credential.
 
     This will get the GSSAPI credential that contains the Kerberos TGT inside
     it. This is used instead of gss_acquire_cred_with_password as the latter
-    does not expose a way to request a forwardable ticket. This way makes it
-    possible to request whatever is needed before making it usable in GSSAPI.
+    does not expose a way to request a forwardable ticket or to retrieve a TGT
+    from a keytab. This way makes it possible to request whatever is needed
+    before making it usable in GSSAPI.
 
     Args:
         username: The username to get the credential for.
         password: The password to use to retrieve the credential.
         forwardable: Whether to request a forwardable credential.
+        is_keytab: Whether password is a keytab or just a password.
 
     Returns:
         gssapi.raw.Creds: The GSSAPI credential for the Kerberos mech.
     """
     ctx = krb5.init_context()
-    princ = krb5.parse_name_flags(ctx, username)
+
+    kt: typing.Optional[krb5.KeyTab] = None
+    princ: typing.Optional[krb5.Principal] = None
+    if is_keytab:
+        kt = krb5.kt_resolve(ctx, password)
+
+        # If the username was not specified get the principal of the first entry.
+        if not username:
+            # The principal handle is deleted once the entry is deallocated. Make sure it is stored in a var before
+            # being copied.
+            first_entry = list(kt)[0]
+            princ = copy.copy(first_entry.principal)
+
+    if not princ:
+        princ = krb5.parse_name_flags(ctx, username)
+
     init_opt = krb5.get_init_creds_opt_alloc(ctx)
 
     if hasattr(krb5, "get_init_creds_opt_set_default_flags"):
@@ -313,27 +388,57 @@ def _kinit(
     if forwardable is not None:
         krb5.get_init_creds_opt_set_forwardable(init_opt, forwardable)
 
-    cred = krb5.get_init_creds_password(ctx, princ, init_opt, password=password)
+    if kt:
+        cred = krb5.get_init_creds_keytab(ctx, princ, init_opt, keytab=kt)
+    else:
+        cred = krb5.get_init_creds_password(ctx, princ, init_opt, password=password)
 
     mem_ccache = krb5.cc_new_unique(ctx, b"MEMORY")
     krb5.cc_initialize(ctx, mem_ccache, princ)
     krb5.cc_store_cred(ctx, mem_ccache, cred)
 
+    return _gss_acquire_cred_from_ccache(mem_ccache, None)
+
+
+def _gss_acquire_cred_from_ccache(
+    ccache: "krb5.CCache",
+    principal: typing.Optional["krb5.Principal"],
+) -> "gssapi.raw.Creds":
+    """Acquire GSSAPI credential from CCache.
+
+    Args:
+        ccache: The CCache to acquire the credential from.
+        principal: The optional principal to acquire the cred for.
+
+    Returns:
+        gssapi.raw.Creds: The GSSAPI credentials from the ccache.
+    """
     # acquire_cred_from is less dangerous than krb5_import_cred which uses a raw pointer to access the ccache. Heimdal
     # has only recently added this API (not in a release as of 2021) so there's a fallback to the latter API.
     if hasattr(gssapi.raw, "acquire_cred_from"):
         kerberos = gssapi.OID.from_int_seq(GSSMech.kerberos.value)
-        gssapi_creds = gssapi.raw.acquire_cred_from(
-            {b"ccache": b"MEMORY:" + mem_ccache.name},
+        name = None
+        if principal:
+            name = gssapi.Name(base=to_text(principal.name), name_type=gssapi.NameType.user)
+
+        ccache_name = ccache.name or b""
+        if ccache.cache_type:
+            ccache_name = ccache.cache_type + b":" + ccache_name
+
+        return gssapi.raw.acquire_cred_from(
+            {b"ccache": ccache_name},
+            name=name,
             mechs=[kerberos],
             usage="initiate",
         ).creds
 
     else:
         gssapi_creds = gssapi.raw.Creds()
-        gssapi.raw.krb5_import_cred(gssapi_creds, cache=mem_ccache.addr)
+        gssapi.raw.krb5_import_cred(
+            gssapi_creds, cache=ccache.addr, keytab_principal=principal.addr if principal else None
+        )
 
-    return gssapi_creds
+        return gssapi_creds
 
 
 class GSSAPIProxy(ContextProxy):
@@ -343,57 +448,58 @@ class GSSAPIProxy(ContextProxy):
     uses the Python gssapi library to interface with the gss_* calls to provider Kerberos, and potentially native
     ntlm/negotiate functionality.
     """
+
     def __init__(
         self,
-        username: typing.Optional[str] = None,
+        username: typing.Optional[typing.Union[str, Credential, typing.List[Credential]]] = None,
         password: typing.Optional[str] = None,
         hostname: typing.Optional[str] = None,
         service: typing.Optional[str] = None,
         channel_bindings: typing.Optional[GssChannelBindings] = None,
         context_req: ContextReq = ContextReq.default,
-        usage: str = 'initiate',
-        protocol: str = 'negotiate',
+        usage: str = "initiate",
+        protocol: str = "negotiate",
         options: NegotiateOptions = NegotiateOptions.none,
-        _is_wrapped: bool = False,
         **kwargs: typing.Any,
     ) -> None:
 
         if not HAS_GSSAPI:
             raise ImportError("GSSAPIProxy requires the Python gssapi library: %s" % GSSAPI_IMP_ERR)
 
-        super(GSSAPIProxy, self).__init__(username, password, hostname, service, channel_bindings, context_req, usage,
-                                          protocol, options, _is_wrapped)
+        credentials = unify_credentials(username, password)
+        super(GSSAPIProxy, self).__init__(
+            credentials, hostname, service, channel_bindings, context_req, usage, protocol, options
+        )
 
         mech_str = {
-            'kerberos': GSSMech.kerberos.value,
-            'negotiate': GSSMech.spnego.value,
-            'ntlm': GSSMech.ntlm.value,
+            "kerberos": GSSMech.kerberos.value,
+            "negotiate": GSSMech.spnego.value,
+            "ntlm": GSSMech.ntlm.value,
         }[self.protocol]
         mech = gssapi.OID.from_int_seq(mech_str)
 
         cred = None
         try:
-            cred = _get_gssapi_credential(mech, self.usage, username=username, password=password,
-                                          context_req=context_req)
+            cred = _get_gssapi_credential(mech, self.usage, credentials=credentials, context_req=context_req)
         except GSSError as gss_err:
             raise SpnegoError(base_error=gss_err, context_msg="Getting GSSAPI credential") from gss_err
 
-        context_kwargs = {}
+        context_kwargs: typing.Dict[str, typing.Any] = {}
 
         if self.channel_bindings:
-            context_kwargs['channel_bindings'] = ChannelBindings(
+            context_kwargs["channel_bindings"] = ChannelBindings(
                 initiator_address_type=self.channel_bindings.initiator_addrtype,
                 initiator_address=self.channel_bindings.initiator_address,
                 acceptor_address_type=self.channel_bindings.acceptor_addrtype,
                 acceptor_address=self.channel_bindings.acceptor_address,
-                application_data=self.channel_bindings.application_data
+                application_data=self.channel_bindings.application_data,
             )
 
-        if self.usage == 'initiate':
-            spn = "%s@%s" % (service.lower() if service else 'host', hostname or 'unspecified')
-            context_kwargs['name'] = gssapi.Name(spn, name_type=gssapi.NameType.hostbased_service)
-            context_kwargs['mech'] = mech
-            context_kwargs['flags'] = self._context_req
+        if self.usage == "initiate":
+            spn = "%s@%s" % (service if service else "host", hostname or "unspecified")
+            context_kwargs["name"] = gssapi.Name(spn, name_type=gssapi.NameType.hostbased_service)
+            context_kwargs["mech"] = mech
+            context_kwargs["flags"] = self._context_req
 
         self._context = gssapi.SecurityContext(creds=cred, usage=self.usage, **context_kwargs)
 
@@ -410,7 +516,7 @@ class GSSAPIProxy(ContextProxy):
     @property
     def client_principal(self) -> typing.Optional[str]:
         # Looks like a bug in python-gssapi where the value still has the terminating null char.
-        return to_text(self._context.initiator_name).rstrip('\x00') if self.usage == 'accept' else None
+        return to_text(self._context.initiator_name).rstrip("\x00") if self.usage == "accept" else None
 
     @property
     def complete(self) -> bool:
@@ -425,13 +531,12 @@ class GSSAPIProxy(ContextProxy):
             return None
 
         return {
-            GSSMech.kerberos.value: 'kerberos',
-            GSSMech.ntlm.value: 'ntlm',
-
+            GSSMech.kerberos.value: "kerberos",
+            GSSMech.ntlm.value: "ntlm",
             # Only set until the negotiate process is complete, will change to one of the above once the context is
             # set up.
-            GSSMech.spnego.value: 'negotiate',
-        }.get(oid, 'unknown: %s' % self._context.mech.dotted_form)
+            GSSMech.spnego.value: "negotiate",
+        }.get(oid, "unknown: %s" % self._context.mech.dotted_form)
 
     @property  # type: ignore
     @wrap_system_error(NativeError, "Retrieving session key")
@@ -447,6 +552,13 @@ class GSSAPIProxy(ContextProxy):
         self._context_attr = int(self._context.actual_flags)
 
         if not self._is_wrapped:
+            # When using NTLM through GSSAPI without it being wrapped by SPNEGO
+            # this function needs to be called in order for gss-ntlmssp to add
+            # the MIC. We don't care about the value just that it would be added
+            # if it could.
+            # https://github.com/gssapi/gss-ntlmssp/issues/69
+            if self.protocol == "ntlm" and not self.complete:
+                _ = self._requires_mech_list_mic
             log.debug("GSSAPI step output: %s", base64.b64encode(out_token or b"").decode())
 
         return out_token
@@ -458,24 +570,25 @@ class GSSAPIProxy(ContextProxy):
         # gss-ntlmssp used to hardcode the conf_state=0 which results in encrpted=False. Because we know it is always
         # sealed we just manually set to True.
         # https://github.com/gssapi/gss-ntlmssp/pull/15
-        encrypted = True if self.negotiated_protocol == 'ntlm' else res.encrypted
+        encrypted = True if self.negotiated_protocol == "ntlm" else res.encrypted
 
         return WrapResult(data=res.message, encrypted=encrypted)
 
     @wrap_system_error(NativeError, "Wrapping IOV buffer")
     def wrap_iov(
         self,
-        iov: typing.List[IOVBuffer],
+        iov: typing.Iterable[IOV],
         encrypt: bool = True,
         qop: typing.Optional[int] = None,
     ) -> IOVWrapResult:
-        iov_buffer = IOV(*self._build_iov_list(iov), std_layout=False)
+        buffers = self._build_iov_list(iov, self._convert_iov_buffer)
+        iov_buffer = GSSIOV(*buffers, std_layout=False)
         encrypted = wrap_iov(self._context, iov_buffer, confidential=encrypt, qop=qop)
 
         return IOVWrapResult(buffers=_create_iov_result(iov_buffer), encrypted=encrypted)
 
     def wrap_winrm(self, data: bytes) -> WinRMWrapResult:
-        if self.negotiated_protocol == 'ntlm':
+        if self.negotiated_protocol == "ntlm":
             # NTLM does not support IOV wrapping, luckily the header is a fixed size so we can split at that.
             wrap_result = self.wrap(data).data
             header = wrap_result[:16]
@@ -484,8 +597,8 @@ class GSSAPIProxy(ContextProxy):
 
         else:
             iov = self.wrap_iov([BufferType.header, data, BufferType.padding]).buffers
-            header = iov[0].data
-            enc_data = iov[1].data
+            header = iov[0].data or b""
+            enc_data = iov[1].data or b""
             padding = iov[2].data or b""
 
         return WinRMWrapResult(header=header, data=enc_data + padding, padding_length=len(padding))
@@ -495,13 +608,17 @@ class GSSAPIProxy(ContextProxy):
         res = gssapi.raw.unwrap(self._context, data)
 
         # See wrap for more info.
-        encrypted = True if self.negotiated_protocol == 'ntlm' else res.encrypted
+        encrypted = True if self.negotiated_protocol == "ntlm" else res.encrypted
 
         return UnwrapResult(data=res.message, encrypted=encrypted, qop=res.qop)
 
     @wrap_system_error(NativeError, "Unwrapping IOV buffer")
-    def unwrap_iov(self, iov: typing.List[IOVBuffer]) -> IOVUnwrapResult:
-        iov_buffer = IOV(*self._build_iov_list(iov), std_layout=False)
+    def unwrap_iov(
+        self,
+        iov: typing.Iterable[IOV],
+    ) -> IOVUnwrapResult:
+        buffers = self._build_iov_list(iov, self._convert_iov_buffer)
+        iov_buffer = GSSIOV(*buffers, std_layout=False)
         res = unwrap_iov(self._context, iov_buffer)
 
         return IOVUnwrapResult(buffers=_create_iov_result(iov_buffer), encrypted=res.encrypted, qop=res.qop)
@@ -518,14 +635,10 @@ class GSSAPIProxy(ContextProxy):
         sasl_desc = _gss_sasl_description(self._context.mech)
 
         # https://github.com/krb5/krb5/blob/f2e28f13156785851819fc74cae52100e0521690/src/lib/gssapi/krb5/gssapi_krb5.c#L686
-        if sasl_desc and sasl_desc == b'Kerberos 5 GSS-API Mechanism':
+        if sasl_desc and sasl_desc == b"Kerberos 5 GSS-API Mechanism":
             # TODO: Should done when self.negotiated_protocol == 'kerberos', above explains why this can't be done yet.
-            iov = self.unwrap_iov([
-                (IOVBufferType.header, header),
-                data,
-                IOVBufferType.data
-            ]).buffers
-            return iov[1].data
+            iov = self.unwrap_iov([(IOVBufferType.header, header), data, IOVBufferType.data]).buffers
+            return iov[1].data or b""
 
         else:
             return self.unwrap(header + data).data
@@ -541,18 +654,16 @@ class GSSAPIProxy(ContextProxy):
     @property
     def _context_attr_map(self) -> typing.List[typing.Tuple[ContextReq, int]]:
         attr_map = [
-            (ContextReq.delegate, 'delegate_to_peer'),
-            (ContextReq.mutual_auth, 'mutual_authentication'),
-            (ContextReq.replay_detect, 'replay_detection'),
-            (ContextReq.sequence_detect, 'out_of_sequence_detection'),
-            (ContextReq.confidentiality, 'confidentiality'),
-            (ContextReq.integrity, 'integrity'),
-
+            (ContextReq.delegate, "delegate_to_peer"),
+            (ContextReq.mutual_auth, "mutual_authentication"),
+            (ContextReq.replay_detect, "replay_detection"),
+            (ContextReq.sequence_detect, "out_of_sequence_detection"),
+            (ContextReq.confidentiality, "confidentiality"),
+            (ContextReq.integrity, "integrity"),
             # Only present when the DCE extensions are installed.
-            (ContextReq.identify, 'identify'),
-
+            (ContextReq.identify, "identify"),
             # Only present with newer versions of python-gssapi https://github.com/pythongssapi/python-gssapi/pull/218.
-            (ContextReq.delegate_policy, 'ok_as_delegate'),
+            (ContextReq.delegate_policy, "ok_as_delegate"),
         ]
         attrs = []
         for spnego_flag, gssapi_name in attr_map:
@@ -572,25 +683,24 @@ class GSSAPIProxy(ContextProxy):
         else:
             return b"\x01" in res
 
-    def _convert_iov_buffer(self, buffer: IOVBuffer) -> typing.Any:
+    def _convert_iov_buffer(self, buffer: IOVBuffer) -> "GSSIOVBuffer":
         buffer_data = None
         buffer_alloc = False
 
         if isinstance(buffer.data, bytes):
             buffer_data = buffer.data
+        elif isinstance(buffer.data, bool):
+            buffer_alloc = buffer.data
         elif isinstance(buffer.data, int):
             # This shouldn't really occur on GSSAPI but is here to mirror what SSPI does.
             buffer_data = b"\x00" * buffer.data
         else:
             auto_alloc = [BufferType.header, BufferType.padding, BufferType.trailer]
+            buffer_alloc = buffer.type in auto_alloc
 
-            buffer_alloc = buffer.data
-            if buffer_alloc is None:
-                buffer_alloc = buffer.type in auto_alloc
-
-        return buffer.type, buffer_alloc, buffer_data
+        return GSSIOVBuffer(IOVBufferType(buffer.type), buffer_alloc, buffer_data)
 
     @wrap_system_error(NativeError, "NTLM reset crypto state")
     def _reset_ntlm_crypto_state(self, outgoing: bool = True) -> None:
-        if self.negotiated_protocol == 'ntlm':
+        if self.negotiated_protocol == "ntlm":
             _gss_ntlmssp_reset_crypto(self._context, outgoing=outgoing)
